package net.nourdine.jp.config;

import java.util.ArrayList;
import java.util.List;
import java.io.File;
import java.io.InputStream;
import java.text.MessageFormat;
import java.util.Properties;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import net.nourdine.jp.dialog.UIDialog;
import net.nourdine.jp.log.JsPackerLogger;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class ConfigReader {

   private static JsPackerLogger log = JsPackerLogger.getInstance();
   public static Properties dialog = UIDialog.getInstace().getProperties();
   private static String SCHEMA_FILENAME = "configFile.xsd";
   private XmlFile conf;
   private ArrayList<String> dependencies = null;

   public ConfigReader(String path) {
      conf = new XmlFile(path);
      getDependencies();
   }

   public void checkConfigFileAgainstDTD() {
      try {
         InputStream schemaIS = this.getClass().getClassLoader().getResourceAsStream("META-INF/" + SCHEMA_FILENAME);
         String schemaLang = "http://www.w3.org/2001/XMLSchema";
         SchemaFactory factory = SchemaFactory.newInstance(schemaLang);
         Schema schema = factory.newSchema(new StreamSource(schemaIS));
         Validator validator = schema.newValidator();
         validator.validate(new StreamSource(new File(this.getXmlConfigPhysicalPath())));
      } catch (Exception e) {
         log.info(MessageFormat.format(dialog.getProperty("config_file_not_validating"), this.getProjectName()));         
         log.error(e.getMessage());
         // e.printStackTrace();
         System.exit(0);
      }
   }

   public String getXmlConfigPhysicalPath() {
      return conf.getPhysicalPath();
   }

   private String getTagValue(Node el) {
      return el.getFirstChild().getNodeValue();
   }

   public String getProjectName() {
      return getTagValue(conf.getDoc().getElementsByTagName("name").item(0));
   }

   public String getJsSrcRoot() {
      return getTagValue(conf.getDoc().getElementsByTagName("srcPath").item(0));
   }

   public String getTargetFolder() {
      return getTagValue(conf.getDoc().getElementsByTagName("buildPath").item(0));
   }

   public String getBuildFileName() {
      return getTagValue(conf.getDoc().getElementsByTagName("buildFileName").item(0));
   }

   public List<String> getJsFiles() {
      ArrayList<String> files = new ArrayList<String>();
      NodeList fileNodes = ((Element) conf.getDoc().getElementsByTagName("files").item(0)).getElementsByTagName("file");
      for (int i = 0; i < fileNodes.getLength(); i++) {
         String fileName = getTagValue(fileNodes.item(i));
         files.add(getJsSrcRoot() + File.separatorChar + fileName);
      }
      return files;
   }

   public final List<String> getDependencies() {
      if (dependencies == null) {
         dependencies = new ArrayList<String>();
         NodeList dependenciesTag = (NodeList) conf.getDoc().getElementsByTagName("dependencies");
         if (dependenciesTag.getLength() > 0) {
            NodeList dependencyNodes = ((Element) dependenciesTag.item(0)).getElementsByTagName("dependency");
            for (int i = 0; i < dependencyNodes.getLength(); i++) {
               String dependencyPath = getTagValue(dependencyNodes.item(i));
               dependencies.add(dependencyPath);
            }
         }
      }
      return dependencies;
   }

   public boolean hasDependencies() {
      return (dependencies.size() > 0);
   }
}
